#include "Behaviors/StateMachine.h"

#include "ColorHist.h"

int min(int x1, int x2) {
    return (x1 <= x2) ? x1 : x2;
}

int max(int x1, int x2) {
    return (x1 >= x2) ? x1 : x2;
}

void doColorHist(const Point &a, const Point &b, const Point &c) {
    int left_bound = min(min(a.coordX(), b.coordX()), c.coordX());
    int right_bound = max(max(a.coordX(), b.coordX()), c.coordX());
    int upper_bound = min(min(a.coordY(), b.coordY()), c.coordY());
    int lower_bound = max(max(a.coordY(), b.coordY()), c.coordY());

    NEW_SKETCH(cam_frame, yuv, VRmixin::sketchFromYUV());
    #define NUM_U_BINS 10
    #define NUM_V_BINS 10
    int histogram[NUM_U_BINS][NUM_V_BINS] = { 0 };

    for(int x = left_bound; x < right_bound; ++x) {
        for(int y = upper_bound; y < lower_bound; ++y) {
            yuv xy_yuv = cam_frame(x, y);
            ++histogram[(xy_yuv.u*NUM_U_BINS)/256][(xy_yuv.v*NUM_V_BINS)/256];
        }
    }

    for(int i = 0; i < NUM_U_BINS; ++i) {
        for(int j = 0; j < NUM_V_BINS; ++j) {
            cout << histogram[i][j] << "\t";
        }
        cout << endl;
    }
}

void printIdColor(const Point &a, const Point &b) {
    NEW_SKETCH(cam_frame, yuv, VRmixin::sketchFromYUV());

    Point center = (a+b)/2;
    yuv id_color = cam_frame(center.coordX(), center.coordY());
    yuv red = cam_frame(a.coordX(), a.coordY());
    cout << "id color: " << (int)id_color.u << ", " << (int)id_color.v << endl;
    cout << "red: " << (int)red.u << ", " << (int)red.v << endl;
}

ColorIdTarget color_ids[] = {
    { 0, 170, 100}, //blue
    { 1, 115, 120}, //green
    { 2, 95, 135}, //yellow
    { 3, 128, 128}, //black
};
#define NUM_COLOR_IDS ((int)(sizeof(color_ids)/sizeof(color_ids[0])))

int getColorIdMatch(yuv id_color) {
    int distances[NUM_COLOR_IDS];
    for(int i = 0; i < NUM_COLOR_IDS; ++i) {
        distances[i] = (id_color.u - color_ids[i].u)*(id_color.u - color_ids[i].u) +
                        (id_color.v - color_ids[i].v)*(id_color.v - color_ids[i].v);
    }

    int min_val = distances[0];
    int min_index = 0;
    for(int i = 1; i < NUM_COLOR_IDS; ++i) {
        if(distances[i] < min_val) {
            min_val = distances[i];
            min_index = i;
        }
    }

    return color_ids[min_index].id;
}

int getIdFromCenter(const Point &a, const Point &b, const Point &c) {
    NEW_SKETCH(cam_frame, yuv, VRmixin::sketchFromYUV());

    Point center = (a + b + c)/3;
    yuv id_color = cam_frame(center.coordX(), center.coordY());

    return getColorIdMatch(id_color);
}

int getIdFromCorner(const Point &vert_a, const Point &vert_b, const Point &loner) {
    NEW_SKETCH(cam_frame, yuv, VRmixin::sketchFromYUV());
    float verta_dist = fabs(loner.coordY() - vert_a.coordY());
    float vertb_dist = fabs(loner.coordY() - vert_b.coordY());
    yuv id_color;

    if(verta_dist <= vertb_dist) {
        id_color = cam_frame(loner.coordX(), vert_b.coordY() + verta_dist);
    } else {
        id_color = cam_frame(loner.coordX(), vert_a.coordY() + vertb_dist);
    }

    return getColorIdMatch(id_color);
}

int getIdFromHist(const Point &a, const Point &b, const Point &c) {
    int left_bound = min(min(a.coordX(), b.coordX()), c.coordX());
    int right_bound = max(max(a.coordX(), b.coordX()), c.coordX());
    int upper_bound = min(min(a.coordY(), b.coordY()), c.coordY());
    int lower_bound = max(max(a.coordY(), b.coordY()), c.coordY());

    NEW_SKETCH(cam_frame, yuv, VRmixin::sketchFromYUV());
    #define NUM_U_BINS 10
    #define NUM_V_BINS 10
    int histogram[NUM_U_BINS][NUM_V_BINS] = { 0 };

    for(int x = left_bound; x < right_bound; ++x) {
        for(int y = upper_bound; y < lower_bound; ++y) {
            yuv xy_yuv = cam_frame(x, y);
            ++histogram[(xy_yuv.u*NUM_U_BINS)/256][(xy_yuv.v*NUM_V_BINS)/256];
        }
    }

    int u_max = 0;
    int v_max = 0;
    int max_val = 0;
    for(int i = 0; i < NUM_U_BINS; ++i) {
        for(int j = 0; j < NUM_V_BINS; ++j) {
            if(histogram[i][j] > max_val) {
                u_max = j;
                v_max = i;
                max_val = histogram[i][j];
            }
        }
    }

    int u_blue = 4;
    int v_blue = 6;

    int u_yellow = 5;
    int v_yellow = 3;

    int u_green = 4;
    int v_green = 4;

    int blue_dist = (u_max - u_blue)*(u_max - u_blue) + (v_max - v_blue)*(v_max - v_blue);
    int yellow_dist = (u_max - u_yellow)*(u_max - u_yellow) + (v_max - v_yellow)*(v_max - v_yellow);
    int green_dist = (u_max - u_green)*(u_max - u_green) + (v_max - v_green)*(v_max - v_green);

    if(blue_dist < yellow_dist) {
        if(blue_dist < green_dist) {
            return 0;
        } else {
            return 1;
        }
    } else {
        if(yellow_dist < green_dist) {
            return 2;
        } else {
            return 1;
        }
    }
}
